#!/usr/bin/env ruby
# 
# Starter sample for text shaping features
# Demonstrate text shaping for Arabic, Hebrew, and Devanagari scripts
# Right-to-left text is reordered according to the Bidi algorithm.
#
# Required software: PDFlib/PDFlib+PDI/PPS 9
# Required data: suitable fonts for the scripts
#

require 'PDFlib'

# This is where the data files are. Adjust as necessary.
searchpath = "../data"
outfile = "starter_shaping.pdf"

llx = 25
lly = 25
urx = 825
ury = 550

headers = [
	"Language",
	"Raw input",
	"Reordered and shaped output"
]

shapingsamples = [
# fontnopt: name of the font for this script
# optlist:  text options
# textflow: can't use Textflow for Bidi text
# language: language name
# text:     sample text

# -------------------------- Arabic --------------------------
{
"fontopt" => "fontname=NotoNaskhArabic-Regular encoding=unicode fallbackfonts={ {fontname=NotoSerif-Regular encoding=unicode} }",
"optlist" => "shaping script=arab",
"textflow" => 0,
"language" => "Arabic",
"text" => "&#x0627;&#x0644;&#x0639;&#x064E;&#x0631;&#x064E;&#x0628;&#x0650;" +
"&#x064A;&#x0629;"
},
{
"fontopt" => "fontname=NotoNaskhArabic-Regular encoding=unicode fallbackfonts={ {fontname=NotoSerif-Regular encoding=unicode} }",
"optlist" => "shaping script=arab",
"textflow" => 0,
"language" => "Arabic",
"text" => "&#x0645;&#x0631;&#x062D;&#x0628;&#x0627;! (Hello)"
},

{
"fontopt" => "fontname=NotoNaskhArabic-Regular encoding=unicode fallbackfonts={ {fontname=NotoSerif-Regular encoding=unicode} }",
"optlist" => "shaping script=arab",
"textflow" => 0,
"language" => "Arabic",
"text" => "&#xFEFF;&#x0627;&#x0644;&#x0645;&#x0627;&#x062F;&#x0629;&#x0020;" +
"&#x0031;&#x0020;&#x064A;&#x0648;&#x0644;&#x062F;&#x0020;&#x062C;" +
"&#x0645;&#x064A;&#x0639;&#x0020;&#x0627;&#x0644;&#x0646;&#x0627;" +
"&#x0633;&#x0020;&#x0623;&#x062D;&#x0631;&#x0627;&#x0631;&#x064B;" +
"&#x0627;&#x0020;&#x0645;&#x062A;&#x0633;&#x0627;&#x0648;&#x064A;" +
"&#x0646;&#x0020;&#x0641;&#x064A;&#x0020;&#x0627;&#x0644;&#x0643;" +
"&#x0631;&#x0627;&#x0645;&#x0629;&#x0020;&#x0648;&#x0627;&#x0644;" +
"&#x062D;&#x0642;&#x0648;&#x0642;&#x002E;&#x0020;"
},

{
"fontopt" => "fontname=NotoNaskhArabic-Regular encoding=unicode fallbackfonts={ {fontname=NotoSerif-Regular encoding=unicode} }",
"optlist" => "shaping script=arab",
"textflow" => 0,
"language" => "Arabic",
"text" => "&#x0648;&#x0642;&#x062F;&#x0020;&#x0648;&#x0647;&#x0628;&#x0648;" +
"&#x0627;&#x0020;&#x0639;&#x0642;&#x0644;&#x0627;&#x064B;&#x0020;" +
"&#x0648;&#x0636;&#x0645;&#x064A;&#x0631;&#x064B;&#x0627;&#x0020;" +
"&#x0648;&#x0639;&#x0644;&#x064A;&#x0647;&#x0645;&#x0020;&#x0623;" +
"&#x0646;&#x0020;&#x064A;&#x0639;&#x0627;&#x0645;&#x0644;&#x0020;" +
"&#x0628;&#x0639;&#x0636;&#x0647;&#x0645;&#x0020;&#x0628;&#x0639;" +
"&#x0636;&#x064B;&#x0627;&#x0020;&#x0628;&#x0631;&#x0648;&#x062D;" +
"&#x0020;&#x0627;&#x0644;&#x0625;&#x062E;&#x0627;&#x0621;&#x002E;"
},

# -------------------------- Hebrew --------------------------
{
"fontopt" => "fontname=NotoSerifHebrew-Regular encoding=unicode fallbackfonts={ {fontname=NotoSerif-Regular encoding=unicode} }",
"optlist" => "shaping script=hebr",
"textflow" => 0,
"language" => "Hebrew",
"text" => "&#x05E2;&#x05B4;&#x05D1;&#x05B0;&#x05E8;&#x05B4;&#x05D9;&#x05EA;"
},

{
"fontopt" => "fontname=NotoSerifHebrew-Regular encoding=unicode fallbackfonts={ {fontname=NotoSerif-Regular encoding=unicode} }",
"optlist" => "shaping script=hebr",
"textflow" => 0,
"language" => "Hebrew",
"text" => "&#x05E1;&#x05E2;&#x05D9;&#x05E3;&#x0020;&#x05D0;&#x002E;&#x0020;" +
"&#x05DB;&#x05DC;&#x0020;&#x05D1;&#x05E0;&#x05D9;&#x0020;&#x05D0;" +
"&#x05D3;&#x05DD;&#x0020;&#x05E0;&#x05D5;&#x05DC;&#x05D3;&#x05D5;" +
"&#x0020;&#x05D1;&#x05E0;&#x05D9;&#x0020;&#x05D7;&#x05D5;&#x05E8;" +
"&#x05D9;&#x05DF;&#x0020;&#x05D5;&#x05E9;&#x05D5;&#x05D5;&#x05D9;" +
"&#x05DD;&#x0020;&#x05D1;&#x05E2;&#x05E8;&#x05DB;&#x05DD;&#x0020;" +
"&#x05D5;&#x05D1;&#x05D6;&#x05DB;&#x05D5;&#x05D9;&#x05D5;&#x05EA;" +
"&#x05D9;&#x05D4;&#x05DD;&#x002E;&#x0020;"
},

{
"fontopt" => "fontname=NotoSerifHebrew-Regular encoding=unicode fallbackfonts={ {fontname=NotoSerif-Regular encoding=unicode} }",
"optlist" => "shaping script=hebr",
"textflow" => 0,
"language" => "Hebrew",
"text" => "&#x05DB;&#x05D5;&#x05DC;&#x05DD;&#x0020;&#x05D7;&#x05D5;&#x05E0;" +
"&#x05E0;&#x05D5;&#x0020;&#x05D1;&#x05EA;&#x05D1;&#x05D5;&#x05E0;" +
"&#x05D4;&#x0020;&#x05D5;&#x05D1;&#x05DE;&#x05E6;&#x05E4;&#x05D5;" +
"&#x05DF;&#x002C;&#x0020;"
},

{
"fontopt" => "fontname=NotoSerifHebrew-Regular encoding=unicode fallbackfonts={ {fontname=NotoSerif-Regular encoding=unicode} }",
"optlist" => "shaping script=hebr",
"textflow" => 0,
"language" => "Hebrew",
"text" => "&#x05DC;&#x05E4;&#x05D9;&#x05DB;&#x05DA;&#x0020;&#x05D7;&#x05D5;" +
"&#x05D1;&#x05D4;&#x0020;&#x05E2;&#x05DC;&#x05D9;&#x05D4;&#x05DD;" +
"&#x0020;&#x05DC;&#x05E0;&#x05D4;&#x05D5;&#x05D2;&#x0020;&#x05D0;" +
"&#x05D9;&#x05E9;&#x0020;&#x05D1;&#x05E8;&#x05E2;&#x05D4;&#x05D5;" +
"&#x0020;&#x05D1;&#x05E8;&#x05D5;&#x05D7;&#x0020;&#x05E9;&#x05DC;" +
"&#x0020;&#x05D0;&#x05D7;&#x05D5;&#x05D4;&#x002E;"
},

# -------------------------- Hindi --------------------------
{
"fontopt" => "fontname=NotoSerifDevanagari-Regular encoding=unicode fallbackfonts={ {fontname=NotoSerif-Regular encoding=unicode} }",
"optlist" => "shaping script=deva",
"textflow" => 1,
"language" => "Hindi",
"text" => "&#x0939;&#x093F;&#x0928;&#x094D;&#x0926;&#x0940;"
},

{
"fontopt" => "fontname=NotoSerifDevanagari-Regular encoding=unicode fallbackfonts={ {fontname=NotoSerif-Regular encoding=unicode} }",
"optlist" => "shaping script=deva advancedlinebreak",
"textflow" => 1,
"language" => "Hindi",
"text" => "&#x0905;&#x0928;&#x0941;&#x091A;&#x094D;&#x091B;&#x0947;&#x0926;" +
"&#x0020;&#x0967;&#x002E;&#x0020;&#x0938;&#x092D;&#x0940;&#x0020;" +
"&#x092E;&#x0928;&#x0941;&#x0937;&#x094D;&#x092F;&#x094B;&#x0902;" +
"&#x0020;&#x0915;&#x094B;&#x0020;&#x0917;&#x094C;&#x0930;&#x0935;" +
"&#x0020;&#x0914;&#x0930;&#x0020;&#x0905;&#x0927;&#x093F;&#x0915;" +
"&#x093E;&#x0930;&#x094B;&#x0902;&#x0020;&#x0915;&#x0947;&#x0020;" +
"&#x092E;&#x093E;&#x092E;&#x0932;&#x0947;&#x0020;&#x092E;&#x0947;" +
"&#x0902;&#x0020;&#x091C;&#x0928;&#x094D;&#x092E;&#x091C;&#x093E;" +
"&#x0924;&#x0020;&#x0938;&#x094D;&#x0935;&#x0924;&#x0928;&#x094D;" +
"&#x0924;&#x094D;&#x0930;&#x0924;&#x093E;&#x0020;&#x0914;&#x0930;" +
"&#x0020;&#x0938;&#x092E;&#x093E;&#x0928;&#x0924;&#x093E;&#x0020;" +
"&#x092A;&#x094D;&#x0930;&#x093E;&#x092A;&#x094D;&#x0924;&#x0020;" +
"&#x0939;&#x0948;&#x0020;&#x0964;&#x0020;&#x0909;&#x0928;&#x094D;" +
"&#x0939;&#x0947;&#x0902;&#x0020;&#x092C;&#x0941;&#x0926;&#x094D;" +
"&#x0918;&#x093F;&#x0020;&#x0914;&#x0930;&#x0020;&#x0905;&#x0928;" +
"&#x094D;&#x0924;&#x0930;&#x093E;&#x0924;&#x094D;&#x092E;&#x093E;" +
"&#x0020;&#x0915;&#x0940;&#x0020;&#x0926;&#x0947;&#x0928;&#x0020;" +
"&#x092A;&#x094D;&#x0930;&#x093E;&#x092A;&#x094D;&#x0924;&#x0020;" +
"&#x0939;&#x0948;&#x0020;&#x0914;&#x0930;&#x0020;&#x092A;&#x0930;" +
"&#x0938;&#x094D;&#x092A;&#x0930;&#x0020;&#x0909;&#x0928;&#x094D;" +
"&#x0939;&#x0947;&#x0902;&#x0020;&#x092D;&#x093E;&#x0908;&#x091A;" +
"&#x093E;&#x0930;&#x0947;&#x0020;&#x0915;&#x0947;&#x0020;&#x092D;" +
"&#x093E;&#x0935;&#x0020;&#x0938;&#x0947;&#x0020;&#x092C;&#x0930;" +
"&#x094D;&#x0924;&#x093E;&#x0935;&#x0020;&#x0915;&#x0930;&#x0928;" +
"&#x093E;&#x0020;&#x091A;&#x093E;&#x0939;&#x093F;&#x090F;&#x0020;" +
"&#x0964;"
},

# -------------------------- Sanskrit --------------------------
{
"fontopt" => "fontname=NotoSerifDevanagari-Regular encoding=unicode fallbackfonts={ {fontname=NotoSerif-Regular encoding=unicode} }",
"optlist" => "shaping script=deva",
"textflow" => 1,
"language" => "Sanskrit",
"text" => "&#x0938;&#x0902;&#x0938;&#x094D;&#x0915;&#x0943;&#x0924;&#x092E;" +
"&#x094D;"
}
]

begin
  # create a new PDFlib object
  p = PDFlib.new

  p.set_option("SearchPath={{" + searchpath +"}}")
  p.set_option("charref=true")

  # This means that formatting and other errors will raise an
  # exception. This simplifies our sample code, but is not
  # recommended for production code.
  #
  p.set_option("errorpolicy=exception")

  # all strings are expected as utf8
  p.set_option("stringformat=utf8")

  # Set an output path according to the name of the topic
  if (p.begin_document(outfile, "") == -1)
    raise "Error: " + p.get_errmsg()
  end

  p.set_info("Creator", "PDFlib starter sample")
  p.set_info("Title", "starter_shaping")

  table = -1
  row = 1
  col = 1

  # Create table header
  headers.each do |header|
      optlist = "fittextline={fontname=NotoSerif-Regular encoding=unicode " +
	  "fontsize=14} colwidth=" + (col==1 ? "10%" : "45%")
      table = p.add_table_cell(table, col, row, header, optlist)
      col += 1
  end
  row += 1

  # Create shaping samples
  shapingsamples.each do |sample|
    col=1

    # Column 1: language name
    optlist = "fittextline={fontname=NotoSerif-Regular encoding=unicode fontsize=12}"
    table = p.add_table_cell(table, col, row, sample["language"], optlist)
    col += 1

    # Column 2: raw text
    optlist = sample["fontopt"] + " fontsize=12 leading=150% alignment=left"
    tf = p.create_textflow(sample["text"], optlist)
    optlist = "margin=4 fittextflow={verticalalign=top} textflow=" +
	tf.to_s + ""
    table = p.add_table_cell(table, col, row, "", optlist)
    col += 1

    # Column 3: shaped and reordered text (Textline or Textflow)
    if (sample["textflow"] != 0)
      optlist = sample["fontopt"] + " " + sample["optlist"] + 
      " fontsize=12 leading=150% alignment=left"
	 
      tf = p.create_textflow(sample["text"], optlist)
      optlist = "margin=4 fittextflow={verticalalign=top} textflow=" +
	  tf.to_s + ""
      table = p.add_table_cell(table, col, row, "", optlist)
    else
      optlist = "fittextline={" +sample["fontopt"] + " "  + 
      sample["optlist"] + " fontsize=12 }"
      table = p.add_table_cell(table, col, row, sample["text"], optlist)
    end
    row += 1
  end

  # ---------- Place the table on one or more pages ----------
  #
  # Loop until all of the table is placed; create new pages
  # as long as more table instances need to be placed.
  #
  begin
      p.begin_page_ext(0, 0, "width=a4.height height=a4.width")

      # Shade every other row; draw lines for all table cells.
      optlist = "header=1 fill={{area=rowodd " +
	  "fillcolor={gray 0.9}}} stroke={{line=other}} "

      # Place the table instance
      result = p.fit_table(table, llx, lly, urx, ury, optlist)

      if (result == "_error")
	  raise "Couldn't place table: " + p.get_errmsg(p)
      end

      p.end_page_ext("")

  end while (result == "_boxfull")

  p.end_document("")

rescue  PDFlibException => pe
  print "PDFlib exception occurred:\n"
  print "[" + pe.get_errnum.to_s + "] " + pe.get_apiname + \
		": " + pe.get_errmsg + "\n"
rescue  Exception => e
  print e.backtrace.join("\n") + "\n" + e.to_s
ensure
  p.delete() if p
end

# vim: sw=4
